@@ -361,12 +361,14 @@ plotAnc <- function(tree, data, i = 1, site.pattern = TRUE, col = NULL,
 
 
 acctran2 <- function(tree, data) {
+  if(!is.binary(tree)) tree <- multi2di(tree)
   tree <- reorder(tree, "postorder")
   edge <- tree$edge
+  data <- subset(data, tree$tip.label)
   f <- init_fitch(data, FALSE, FALSE, m=2L)
   psc_node <- f$pscore_node(edge)
   tmp <- reorder(tree)$edge
-  tmp <- tmp[tmp[,2]>Ntip(tree),]
+  tmp <- tmp[tmp[,2]>Ntip(tree), ,drop=FALSE]
   f$traverse(edge)
   if(length(tmp)>0)f$acctran_traverse(tmp)
   psc <- f$pscore_acctran(edge)
